@@ -18,7 +18,7 @@ from typing import Callable
 import numpy as np
 
 from trident.backend.common import to_list, format_time, get_terminal_size, get_session, get_backend, \
-    PrintException, OrderedDict, split_path, sanitize_path, Signature, adaptive_format, dtype, cyan_color
+    PrintException, OrderedDict, split_path, sanitize_path, adaptive_format, dtype, cyan_color
 from trident.backend.opencv_backend import array2image
 from trident.backend.tensorspec import *
 from trident.callbacks import LambdaCallback, UnfreezeModelCallback
@@ -306,8 +306,10 @@ class ModelBase(object):
                     img_data = fc(img_data)
             img_data = np.expand_dims(image_backend_adaption(img_data), 0)
             if self.input_spec is None:
-                self.signature.inputs['input'] = TensorSpec(shape=tensor_to_shape(to_tensor(img_data), need_exclude_batch_axis=False), dtype=dtype.float32,
-                                                            object_type=ObjectType.rgb, name='input')
+                self._signature = get_signature(self._model)
+                self._signature.inputs[self._signature.inputs.key_list[0]].shape = tensor_to_shape(to_tensor(img_data), need_exclude_batch_axis=False)
+                self._signature.inputs[self._signature.inputs.key_list[0]].object_type = ObjectType.rgb
+                self._signature.inputs[self._signature.inputs.key_list[0]].dtype = dtype.float32
 
             return img_data
         else:
@@ -405,6 +407,10 @@ class ModelBase(object):
     def __setattr__(self, name, value):
         if name in ['_input_shape', '_output_shape', '_class_names', 'class_names', 'output_fn', 'optimizer']:
             object.__setattr__(self, name, value)
+        elif name in ['trainable']:
+            _model = self.__dict__['_model']
+            if hasattr(_model, 'trainable'):
+                _model.trainable = value
 
         elif name in ['_model']:
             object.__setattr__(self, '_model', value)
@@ -423,7 +429,7 @@ class ModelBase(object):
 
             if name == 'signature' or name == '_signature':
                 _model = self.__dict__['_model']
-                if _model is not None :
+                if _model is not None:
                     object.__setattr__(self.__dict__['_model'], "_" + name, value)
 
             if 'training_context' in self.__dict__ and name in self.__dict__['training_context']:
@@ -461,7 +467,7 @@ class ModelBase(object):
     def with_metric(self, metric, name='', print_only=False, **kwargs):
         return self
 
-    def with_regularizer(self, reg, **kwargs):
+    def with_regularizer(self, reg, reg_weight=None, **kwargs):
         return self
 
     def with_constraint(self, constraint, **kwargs):
@@ -558,14 +564,6 @@ class ModelBase(object):
     def rebinding_input_output(self, input_shape):
         pass
 
-    def do_on_training_start(self):
-        for callback in self.training_context['callbacks']:
-            callback.on_training_start(self.training_context)
-
-    def do_on_training_end(self):
-        for callback in self.training_context['callbacks']:
-            callback.on_training_end(self.training_context)
-
     def do_on_epoch_start(self):
         for callback in self.callbacks:
             callback.on_epoch_start(self.training_context)
@@ -645,15 +643,16 @@ class ModelBase(object):
                     data[k] = v
 
             else:
-                self.train_data[self.signature.outputs.key_list[0]] = output
+                data[self.signature.outputs.key_list[0]] = output
                 if is_training and self.use_output_as_loss:
                     this_loss = output.sum()
-                    self.training_context['losses'].collect(signature.outputs.key_list[0], self.training_context['steps'], this_loss)
                     self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
+                    self.training_context['losses'].collect(signature.outputs.key_list[0], self.training_context['steps'], to_scalar(this_loss.copy()))
 
     def do_on_loss_calculation_start(self):
         for callback in self.callbacks:
             callback.on_loss_calculation_start(self.training_context)
+
     def do_on_loss_calculation_end(self):
         for callback in self.callbacks:
             callback.on_loss_calculation_end(self.training_context)
@@ -663,7 +662,6 @@ class ModelBase(object):
         # confirm singleton
         # output=unpack_singleton(output)
 
-
         # losss
         for k, v in self._losses.items():
             if not hasattr(v, 'start_epoch') or (hasattr(v, 'start_epoch') and v.start_epoch <= self.training_context['current_epoch']):
@@ -703,7 +701,6 @@ class ModelBase(object):
 
         self.do_on_loss_calculation_end()
 
-
     def do_calculate_regularizations(self):
         # regularizer
         for k, v in self._regs.items():
@@ -744,9 +741,8 @@ class ModelBase(object):
     def do_calculate_metrics(self):
         # ON_EVALUATION_START
         self.do_on_metrics_evaluation_start()
-
         for k, v in self._metrics.items():
-            print_only = not (getattr(v, 'print_only') if hasattr(v, 'print_only') else False)
+            print_only = (getattr(v, 'print_only') if hasattr(v, 'print_only') else False)
             if not print_only:
                 self.training_context['metrics'].regist(k)
             self.training_context['tmp_metrics'].regist(k)
@@ -788,10 +784,10 @@ class ModelBase(object):
     def save_model(self, save_path=None):
         return NotImplemented
 
-    def save_onnx(self, save_path=None, dynamic_axes=None,**kwargs):
+    def save_onnx(self, save_path=None, dynamic_axes=None, **kwargs):
         return NotImplemented
 
-    def save_history(self, save_path=None,**kwargs):
+    def save_history(self, save_path=None, **kwargs):
 
         default_file_name = '{0}_history_{1}.json_'.format(self._model.name, self.training_context['execution_id'])
         save_path = self.training_context['save_path']
@@ -808,14 +804,13 @@ class ModelBase(object):
             f.write(jstring)
             shutil.copy(save_path, save_path.replace('.json_', '.json'))
 
-    def save_weights(self, save_path=None,**kwargs):
+    def save_weights(self, save_path=None, **kwargs):
         if save_path is not None:
             pass
         else:
             save_path = self.training_context['save_path']
 
-
-    def load_model(self, file_path,**kwargs):
+    def load_model(self, file_path, **kwargs):
         return NotImplemented
 
     def print_batch_progress(self, print_batch_progress_frequency):
@@ -931,7 +926,6 @@ class ModelBase(object):
             if (not self.training_context['skip_reset_total_loss']):  # and (not (self.training_context['steps'] + 1) % self.accumulation_steps != 0):
                 self.training_context['current_loss'] = to_tensor(0.0, requires_grad=True)
 
-
             if 'skip_generate_output' not in self.training_context or self.training_context['skip_generate_output'] == False:
                 try:
                     if self.output_fn is not None and callable(self.output_fn):
@@ -948,7 +942,6 @@ class ModelBase(object):
             self.do_calculate_losses()
             self.do_calculate_regularizations()
 
-            self.do_pre_optimization_step()
             self.do_gradient_update(log_gradients and is_collect_data)
 
             if isinstance(self._model, Layer) and any_abnormal_number(self._model):
@@ -962,14 +955,13 @@ class ModelBase(object):
                 elif is_tensor(self._model):
                     self.log_weight(weghts=self._model)
 
-            self.do_calculate_metrics()
-
             if is_out_sample_evaluation == True and self.test_data is not None and len(self.test_data) > 0 and self.training_context['stop_update'] < 1:
                 if self.output_fn is not None and callable(self.output_fn):
                     self.output_fn(self.training_context, is_training=False, is_autocast_enabled=self.is_autocast_enabled)
                 else:
                     self.do_calculate_forward(is_training=False)
 
+            self.do_calculate_metrics()
             # aggregate the step loss and metrics
 
             if is_print_batch_progress or is_print_epoch_progress or is_epoch_end:
@@ -979,7 +971,7 @@ class ModelBase(object):
 
                     # check
                     if len(steps) > 0:
-                        values = np.mean(to_numpy(values))
+                        values = to_scalar(values)
                         self.training_context['metrics'].collect(k, self.training_context['steps'], values)
                 self.training_context['tmp_metrics'].reset()
 
@@ -990,7 +982,6 @@ class ModelBase(object):
             # print batch progresss
             if is_print_batch_progress:
 
-
                 self.print_batch_progress(self.training_context['print_batch_progress_frequency'])
 
                 self.training_context['print_batch_progress_frequency'] = 1
@@ -1003,7 +994,7 @@ class ModelBase(object):
                 for k in self.training_context['out_sample_metrics'].get_keys():
                     test_steps, test_values = self.training_context['out_sample_metrics'].get_series(k)
                     metric_value = test_values[-1]
-                    history_metric_value = np.array(test_values).mean()
+                    history_metric_value = to_scalar(test_values[-1])
 
                     verbose.append('{0}: {1}'.format(k, adaptive_format(metric_value)))
                 out_sample_evaluation_str = cyan_color(self.training_context['model_name'] + ' ' + 'out-of-sample evaluation: ' + ','.join(verbose))
