@@ -166,8 +166,7 @@ class TensorSpec(object):
                  object_type: Optional[ObjectType] = None,
                  optional=False,
                  default=None,
-
-                 name=None):
+                 name=None,**kwargs):
         self._dtype = dtype if dtype is not None else None
         self._shape_tuple = None
         self.object_type = object_type
