@@ -17,6 +17,7 @@ from trident.backend.load_backend import *
 from trident.backend.pillow_backend import image2array
 from trident.callbacks.callback_base import CallbackBase
 from trident.data.dataset import MaskDataset, ImageDataset, ZipDataset
+from trident.data.image_common import image_backend_adaption
 from trident.data.mask_common import label2color
 from trident.misc.ipython_utils import is_in_ipython, is_in_colab
 from trident.misc.visualization_utils import *
@@ -252,13 +253,16 @@ class SegTileImageCallback(VisualizationCallbackBase):
         self.palette = palette
         self.tile_image_name_prefix = name_prefix
         self.reverse_image_transform = reverse_image_transform
-        self.background = np.expand_dims(np.expand_dims(to_numpy(background), 0), 0)
+
+        self.background =to_numpy(background)
 
     def plot_tile_image(self, training_context):
         axis = 1
         if get_backend() == 'tensorflow':
             axis = -1
-
+        new_shape = [1] * 4
+        new_shape[axis] = 3
+        background =np.reshape(to_numpy(self.background), new_shape)
         tile_images_list = []
         input = None
         target = None
@@ -277,7 +281,7 @@ class SegTileImageCallback(VisualizationCallbackBase):
                 if is_label_mask:
                     output = to_numpy(argmax(model(input), axis=axis))
                 else:
-                    output = to_numpy(expand_dims(cast(argmax(model(input), axis=axis), input.dtype), axis=-1))
+                    output = to_numpy(expand_dims(cast(argmax(model(input), axis=axis), input.dtype), axis=axis))
 
                 model.train()
 
@@ -305,7 +309,8 @@ class SegTileImageCallback(VisualizationCallbackBase):
         input_arr = []
         input = to_numpy(input)
         for i in range(len(input)):
-            input_arr.append(self.reverse_image_transform(input[i]))
+            input_arr.append(image_backend_adaption(self.reverse_image_transform(input[i])))
+        input_arr=np.stack(input_arr,axis=0)
         # input_arr=np.asarray(input_arr)
         tile_images_list.append(input_arr)
 
@@ -319,13 +324,11 @@ class SegTileImageCallback(VisualizationCallbackBase):
                 if get_backend() == 'tensorflow':
                     target_arr = np.expand_dims(target, -1)
                 else:
-                    target_arr = np.expand_dims(target, 1)
+                    target_arr = np.expand_dims(target, axis=axis)
 
             if 'alpha' not in data:
                 target_arr[target_arr > 0] = 1
 
-            background = np.ones_like(target_arr) * self.background
-
             tile_images_list.append(target_arr * input_arr + (1 - target_arr) * background)
 
             tile_images_list.append(output_arr * input_arr + (1 - output_arr) * background)
@@ -427,7 +430,7 @@ class PlotLossMetricsCallback(VisualizationCallbackBase):
             self.training_items = training_context['training_items']
 
     def on_overall_batch_end(self, training_context):
-        if not self.is_inplace:
+        if not self.is_inplace  and training_context['steps']>10:
             if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
                     self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
                 if is_in_ipython() and self.counter == self.clean_ipython_output_frequency:
