@@ -177,7 +177,7 @@ class _Context:
         self.image_backend = 'opencv'
         self.epoch_equivalent = 1000
         self.floatx = 'float32'
-        self.epsilon = 1e-8
+        self.epsilon = 1e-7
         self.working_directory = os.getcwd()
         self.plateform = self._get_plateform()
         self.numpy_print_format = '{0:.4e}'
@@ -203,6 +203,7 @@ class _Context:
             self.working_directory = os.environ['TRIDENT_WORKING_DIR']
             os.chdir(os.environ['TRIDENT_WORKING_DIR'])
 
+
         if 'TRIDENT_BACKEND' in os.environ:
             if self.backend != os.environ['TRIDENT_BACKEND']:
                 self.backend = os.environ['TRIDENT_BACKEND']
@@ -279,8 +280,13 @@ class _Context:
             self._thread_local_info.data_providers=OrderedDict()
         self._thread_local_info.data_providers[getattr(data_provider,'uuid')]=data_provider
 
-    def get_data_provider(self):
-        return list(self._thread_local_info.data_providers.values())
+    def get_data_provider(self,name=None):
+        if name is None:
+            return list(self._thread_local_info.data_providers.values())
+        else:
+            for dp in self._thread_local_info.data_providers.values():
+                if dp.name==name:
+                    return dp
 
     def regist_resources(self,resource_name,resource ):
         if not hasattr(self._thread_local_info,'resources'):
