@@ -27,6 +27,7 @@ try:
 except ImportError:
     from six.moves.urllib.request import urlretrieve
 from trident.backend.common import *
+from trident.backend.tensorspec import *
 from trident.backend.opencv_backend import *
 from trident.data.text_common import *
 from trident.data.image_common import *
@@ -49,7 +50,7 @@ class ImageDataProvider(object):
     supporting integer indexing in range from 0 to len(self) exclusive.
     """
 
-    def __init__(self, dataset_name='', traindata=None, testdata=None, minibatch_size=8, mode='tuple', **kwargs):
+    def __init__(self, dataset_name='', traindata=None, testdata=None, batch_size=8, mode='tuple', **kwargs):
         self.__name__ = dataset_name
         self.uuid = uuid.uuid4().node
         self.traindata = traindata
@@ -63,8 +64,7 @@ class ImageDataProvider(object):
         else:
             raise ValueError("Valid mode should be tuple or dict ")
 
-        self._minibatch_size = minibatch_size
-        self.is_flatten = bool(kwargs['is_flatten']) if 'is_flatten' in kwargs else False
+        self._batch_size = batch_size
         self.__default_language__ = 'en-us'
         if len(self._class_names) > 0:
             if _locale in self._class_names:
@@ -83,8 +83,8 @@ class ImageDataProvider(object):
         self._image_transform_funcs = []
         self._label_transform_funcs = []
         self._paired_transform_funcs = []
-        self.spatial_transform_funcs = []
-        cxt=context._context()
+        self._batch_transform_funcs = []
+        cxt = context._context()
         cxt.regist_data_provider(self)
 
     @property
@@ -116,16 +116,16 @@ class ImageDataProvider(object):
             return []
 
     @property
-    def minibatch_size(self):
-        return self._minibatch_size
+    def batch_size(self):
+        return self._batch_size
 
-    @minibatch_size.setter
-    def minibatch_size(self, value):
-        self._minibatch_size = value
-        if self.traindata is not None and hasattr(self.traindata, '_minibatch_size'):
-            self.traindata.minibatch_size = self._minibatch_size
-        if self.testdata is not None and hasattr(self.testdata, '_minibatch_size'):
-            self.testdata.minibatch_size = self._minibatch_size
+    @batch_size.setter
+    def batch_size(self, value):
+        self._batch_size = value
+        if self.traindata is not None and hasattr(self.traindata, '_batch_size'):
+            self.traindata.batch_size = self._batch_size
+        if self.testdata is not None and hasattr(self.testdata, '_batch_size'):
+            self.testdata.batch_size = self._batch_size
 
     @property
     def palette(self):
@@ -144,13 +144,19 @@ class ImageDataProvider(object):
     def image_transform_funcs(self, value):
         self._image_transform_funcs = value
         if self.traindata is not None and hasattr(self.traindata.data, 'transform_funcs'):
-            self.traindata.data.transform_funcs =copy.deepcopy(value)
 
-            self.traindata.update_data_template()
+            dss = self.traindata.get_datasets()
+            for ds in dss:
+                if isinstance(ds, ImageDataset):
+                    ds.transform_funcs = copy.deepcopy(value)
+            self.traindata.update_signature()
         if self.testdata is not None and len(self.testdata.data) > 0 and hasattr(self.testdata.data, 'transform_funcs'):
-            self.testdata.data.transform_funcs =copy.deepcopy(value)
+            dss_t = self.testdata.get_datasets()
+            for ds in dss_t:
+                if isinstance(ds, ImageDataset):
+                    ds.transform_funcs = copy.deepcopy(value)
 
-            self.testdata.update_data_template()
+            self.testdata.update_signature()
 
     def image_transform(self, img_data):
         if img_data.ndim == 4:
@@ -174,7 +180,7 @@ class ImageDataProvider(object):
             fn = self.image_transform_funcs[-1 - i]
             if (inspect.isfunction(fn) and fn.__qualname__ == 'normalize.<locals>.img_op') or (isinstance(fn, Transform) and fn.name == 'normalize'):
                 return_list.append(Unnormalize(fn.mean, fn.std))
-        #return_list.append(array2image)
+        # return_list.append(array2image)
         return return_list
 
     def reverse_image_transform(self, img_data: np.ndarray):
@@ -186,7 +192,7 @@ class ImageDataProvider(object):
             # if img_data.ndim>=2:
             for fc in self.reverse_image_transform_funcs:
                 img_data = fc(img_data)
-            #img_data = reverse_image_backend_adaption(img_data)
+            # img_data = reverse_image_backend_adaption(img_data)
 
         return img_data
 
@@ -199,10 +205,10 @@ class ImageDataProvider(object):
         self._label_transform_funcs = value
         if self.traindata is not None and hasattr(self.traindata.label, 'label_transform_funcs'):
             self.traindata.label.label_transform_funcs = self._label_transform_funcs
-            self.traindata.update_data_template()
+            self.traindata.update_signature()
         if self.testdata is not None and hasattr(self.testdata.label, 'label_transform_funcs'):
             self.testdata.label.label_transform_funcs = self._label_transform_funcs
-            self.testdata.update_data_template()
+            self.testdata.update_signature()
 
     @property
     def paired_transform_funcs(self):
@@ -213,20 +219,54 @@ class ImageDataProvider(object):
         self._paired_transform_funcs = value
 
         if self.traindata is not None and hasattr(self.traindata, 'paired_transform_funcs'):
-            self.traindata.paired_transform_funcs =copy.deepcopy(value)
+            self.traindata.paired_transform_funcs = copy.deepcopy(value)
 
-
-            self.traindata.update_data_template()
+            self.traindata.update_signature()
 
         if self.testdata is not None and hasattr(self.testdata, 'paired_transform_funcs'):
-            self.testdata.paired_transform_funcs =copy.deepcopy(value)
-            self.testdata.update_data_template()
+            self.testdata.paired_transform_funcs = copy.deepcopy(value)
+            self.testdata.update_signature()
+
+    @property
+    def batch_transform_funcs(self):
+        return self._batch_transform_funcs
+
+    @batch_transform_funcs.setter
+    def batch_transform_funcs(self, value):
+        self._batch_transform_funcs = value
+        self.traindata.update_signature()
+        self.traindata.batch_sampler._batch_transform_funcs = value
+        if self.testdata is not None:
+            self.testdata.update_signature()
+
+    def batch_transform(self, batchdata):
+        if hasattr(self, '_batch_transform_funcs') and len(self._batch_transform_funcs) > 0:
+
+            if isinstance(batchdata, tuple):
+                new_batchdata = OrderedDict()
+                for ds in self.traindata.get_datasets():
+                    new_batchdata[ds.element_spec] = None
+
+            if isinstance(batchdata, OrderedDict):
+                if not all([isinstance(k, TensorSpec) for k in batchdata.key_list]):
+                    new_batchdata = copy.deepcopy(self.traindata.data_template)
+                    for i in range(len(batchdata)):
+                        new_batchdata[new_batchdata.key_list[i]] = batchdata[batchdata.key_list[i]]
+                    batchdata = new_batchdata
+
+                for trans in self._batch_transform_funcs:
+                    batchdata = trans(batchdata)
+                return batchdata
+            else:
+                return batchdata
+
+        else:
+            return batchdata
 
     @property
     def class_names(self):
         return self._class_names
 
-
     @class_names.setter
     def class_names(self, value):
         self._class_names = value
@@ -235,38 +275,40 @@ class ImageDataProvider(object):
         if self.testdata is not None and hasattr(self.testdata.label, 'class_names'):
             self.testdata.label.class_names = self._class_names
 
-    def preview_images(self, key=None,is_concate=True):
-        if not isinstance(self.traindata.data,ImageDataset):
+    def preview_images(self, key=None, is_concate=True):
+        if not isinstance(self.traindata.data, ImageDataset):
             return None
         else:
-            if key is None and isinstance(self.traindata.data,ImageDataset):
-                data= self.next()
-                data=enforce_singleton(data)
+            if key is None and isinstance(self.traindata.data, ImageDataset):
+                data = self.next()
+                if isinstance(data, OrderedDict):
+                    data = data.value_list
+                data = enforce_singleton(data)
                 data = self.reverse_image_transform(data)
                 if is_concate:
-                    data = np.concatenate([img for img in data], axis=-1 if data[0].ndim==2 or (data[0].ndim==3 and data[0].shape[0] in [1,3,4]) else -2)
+                    data = np.concatenate([img for img in data], axis=-1 if data[0].ndim == 2 or (data[0].ndim == 3 and data[0].shape[0] in [1, 3, 4]) else -2)
                     return array2image(data)
                 else:
                     return [array2image(img) for img in data]
             elif isinstance(key, slice):
                 start = key.start if key.start is not None else 0
                 stop = key.stop
-                results=[]
-                for k in range(start,stop,1):
-                    img=self.traindata.data.__getitem__(k)
+                results = []
+                for k in range(start, stop, 1):
+                    img = self.traindata.data.__getitem__(k)
 
                     if isinstance(img, np.ndarray):
                         for fc in self.image_transform_funcs:
-                            if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption and  fc is not Normalize and fc is not normalize:
+                            if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption and fc is not Normalize and fc is not normalize:
                                 img = fc(img)
                     results.append(img)
                 if is_concate:
-                    results = np.concatenate(results, axis=-1 if results[0].ndim==2 or (results[0].ndim==3 and results[0].shape[0] in [1,3,4]) else -2)
+                    results = np.concatenate(results, axis=-1 if results[0].ndim == 2 or (results[0].ndim == 3 and results[0].shape[0] in [1, 3, 4]) else -2)
                     return array2image(results)
                 else:
                     return [array2image(img) for img in results]
             elif isinstance(key, int):
-                img=self.traindata.data.__getitem__(key)
+                img = self.traindata.data.__getitem__(key)
                 if isinstance(img, np.ndarray):
                     for fc in self.image_transform_funcs:
                         if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption and fc is not Normalize and fc is not normalize:
@@ -274,25 +316,22 @@ class ImageDataProvider(object):
                 return array2image(img)
 
     def label_statistics(self):
-        if isinstance(self.traindata.label,LabelDataset):
+        if isinstance(self.traindata.label, LabelDataset):
             unique, counts = np.unique(np.array(self.traindata.label.items), return_counts=True)
-            max_len=builtins.max([get_string_actual_length(item) for item in self.class_names[self.__default_language__]])+5
+            max_len = builtins.max([get_string_actual_length(item) for item in self.class_names[self.__default_language__]]) + 5
             for i in range(len(unique)):
-                class_names=''.join([' ']*max_len) if self.class_names is None or len(self.class_names) == 0 else self.index2label(unique[i])+''.join([' ']*(max_len-get_string_actual_length(self.index2label(unique[i]))))
-                bar=['█']*int(builtins.round(50*counts[i] / float(len(self.traindata.label.items))))
-                print('{0:<10} {1} {2:<50} {3:,} ({4:.3%})'.format(unique[i],class_names,''.join(bar),counts[i],counts[i]/float(len(self.traindata.label.items))))
-
-
-
+                class_names = ''.join([' '] * max_len) if self.class_names is None or len(self.class_names) == 0 else self.index2label(unique[i]) + ''.join(
+                    [' '] * (max_len - get_string_actual_length(self.index2label(unique[i]))))
+                bar = ['█'] * int(builtins.round(50 * counts[i] / float(len(self.traindata.label.items))))
+                print('{0:<10} {1} {2:<50} {3:,} ({4:.3%})'.format(unique[i], class_names, ''.join(bar), counts[i], counts[i] / float(len(self.traindata.label.items))))
 
     def _next_index(self):
         return self.__next__()
 
     def __iter__(self):
+
         if self.scenario == 'test' and self.testdata is not None:
             return self.testdata._sample_iter
-
-
         else:
             return self.traindata._sample_iter
 
@@ -311,7 +350,6 @@ class ImageDataProvider(object):
         else:
             return self.traindata.next()
 
-
     def __next__(self):
         if self.scenario == 'test' and self.testdata is not None:
             return next(self.testdata)
@@ -319,19 +357,18 @@ class ImageDataProvider(object):
             return next(self.traindata)
 
     def next_train(self):
-        return self.traindata.next()
-
+        return self.batch_transform(self.traindata.next())
 
     def next_test(self):
         if self.testdata is not None:
-            return self.testdata.next()
+            return self.batch_transform(self.testdata.next())
 
         else:
             return None
 
-    def get_all_data(self, is_shuffle=False,topk=-1):
+    def get_all_data(self, is_shuffle=False, topk=-1):
         idxes = np.arange(len(self.traindata.data))
-        if is_shuffle :
+        if is_shuffle:
             np.random.shuffle(idxes)
         data = []
         if topk == -1:
@@ -340,7 +377,6 @@ class ImageDataProvider(object):
         for i in range(topk):
             data.append(self.traindata.data.__getitem__(idxes[i]))
 
-
         return data
 
     def binding_class_names(self, class_names=None, language=None):
@@ -387,18 +423,15 @@ class ImageDataProvider(object):
 
     def __setattr__(self, name: str, value) -> None:
         object.__setattr__(self, name, value)
-        if name=='mode':
-            if isinstance(self.traindata,Iterator):
-                self.traindata.mode=value
-                self.traindata.batch_sampler.mode=value
-            if isinstance(self.testdata,Iterator):
-                self.testdata.mode=value
+        if name == 'mode':
+            if isinstance(self.traindata, Iterator):
+                self.traindata.mode = value
+                self.traindata.batch_sampler.mode = value
+            if isinstance(self.testdata, Iterator):
+                self.testdata.mode = value
                 self.testdata.batch_sampler.mode = value
 
 
-
-
-
 DataProvider = ImageDataProvider
 
 
@@ -410,8 +443,10 @@ class TextSequenceDataProvider(object):
     supporting integer indexing in range from 0 to len(self) exclusive.
     """
 
-    def __init__(self, dataset_name='', traindata=None, testdata=None, minibatch_size=8, mode='tuple', **kwargs):
+    def __init__(self, dataset_name='', traindata=None, testdata=None, batch_size=8,sequence_length=64, mode='tuple', **kwargs):
         self.__name__ = dataset_name
+        self.sequence_length = sequence_length
+        self.uuid = uuid.uuid4().node
         if mode in ['tuple', 'dict']:
             self.mode = mode
         else:
@@ -424,8 +459,7 @@ class TextSequenceDataProvider(object):
         self.scenario = 'train'
         self._class_names = {}
 
-        self._minibatch_size = minibatch_size
-        self.is_flatten = bool(kwargs['is_flatten']) if 'is_flatten' in kwargs else False
+        self._batch_size = batch_size
         self.__default_language__ = 'en-us'
         if len(self._class_names) > 0:
             if _locale in self._class_names:
@@ -435,6 +469,15 @@ class TextSequenceDataProvider(object):
                     self.__default_language__ = k
                     break
 
+        if isinstance(self.traindata ,Iterator):
+            for ds in self.traindata.get_datasets():
+                if isinstance(ds,TextSequenceDataset):
+                    ds.sequence_length=self.sequence_length
+        if isinstance(self.testdata, Iterator):
+            for ds in self.testdata.get_datasets():
+                if isinstance(ds,TextSequenceDataset):
+                    ds.sequence_length=self.sequence_length
+
         self._idx2lab = {}
         self._lab2idx = {}
 
@@ -444,7 +487,9 @@ class TextSequenceDataProvider(object):
         self._text_transform_funcs = []
         self._label_transform_funcs = []
         self._paired_transform_funcs = []
-        self.spatial_transform_funcs = []
+        self._batch_transform_funcs = []
+        cxt = context._context()
+        cxt.regist_data_provider(self)
 
     @property
     def signature(self):
@@ -475,16 +520,25 @@ class TextSequenceDataProvider(object):
             return []
 
     @property
-    def minibatch_size(self):
-        return self._minibatch_size
+    def batch_sampler(self):
+        if self.scenario == 'test' and self.testdata is not None:
+            return self.testdata.batch_sampler
+        elif self.traindata is not None:
+            return self.traindata.batch_sampler
+        else:
+            return []
 
-    @minibatch_size.setter
-    def minibatch_size(self, value):
-        self._minibatch_size = value
-        if self.traindata is not None and hasattr(self.traindata, '_minibatch_size'):
-            self.traindata.minibatch_size = self._minibatch_size
-        if self.testdata is not None and hasattr(self.testdata, '_minibatch_size'):
-            self.testdata.minibatch_size = self._minibatch_size
+    @property
+    def batch_size(self):
+        return self._batch_size
+
+    @batch_size.setter
+    def batch_size(self, value):
+        self._batch_size = value
+        if self.traindata is not None and hasattr(self.traindata, '_batch_size'):
+            self.traindata.batch_size = self._batch_size
+        if self.testdata is not None and hasattr(self.testdata, '_batch_size'):
+            self.testdata.batch_size = self._batch_size
 
     @property
     def palette(self):
@@ -579,6 +633,42 @@ class TextSequenceDataProvider(object):
         if self.testdata is not None and hasattr(self.testdata, 'paired_transform_funcs'):
             self.testdata.paired_transform_funcs = value
 
+    @property
+    def batch_transform_funcs(self):
+        return self._batch_transform_funcs
+
+    @batch_transform_funcs.setter
+    def batch_transform_funcs(self, value):
+        self._batch_transform_funcs = value
+        self.traindata.update_signature()
+        self.traindata.batch_sampler._batch_transform_funcs = value
+        if self.testdata is not None:
+            self.testdata.update_signature()
+
+    def batch_transform(self, batchdata):
+        if hasattr(self, '_batch_transform_funcs') and len(self._batch_transform_funcs) > 0:
+
+            if isinstance(batchdata, tuple):
+                new_batchdata = copy.deepcopy(self.traindata.data_template)
+                for i in range(len(batchdata)):
+                    new_batchdata[new_batchdata.key_list[i]] = batchdata[i]
+                batchdata = new_batchdata
+            if isinstance(batchdata, OrderedDict):
+                if not all([isinstance(k, TensorSpec) for k in batchdata.key_list]):
+                    new_batchdata = copy.deepcopy(self.traindata.data_template)
+                    for i in range(len(batchdata)):
+                        new_batchdata[new_batchdata.key_list[i]] = batchdata[batchdata.key_list[i]]
+                    batchdata = new_batchdata
+
+                for trans in self._batch_transform_funcs:
+                    batchdata = trans(batchdata)
+                return batchdata
+            else:
+                return batchdata
+
+        else:
+            return batchdata
+
     def _next_index(self):
         return self.__next__()
 
@@ -611,7 +701,6 @@ class TextSequenceDataProvider(object):
     def next_train(self):
         return self.traindata.next()
 
-
     def next_test(self):
         if self.testdata is not None:
             return self.testdata.next()
@@ -636,26 +725,54 @@ class TextSequenceDataProvider(object):
         if text_data in text2indexdict:
             return text2indexdict[text_data]
         else:
-            return text2indexdict['<unknown/>']
+            return text2indexdict['[UNK]']
 
     def index2label(self, idx: int):
         index2textdict = None
         if self.scenario == 'test' and self.testdata is not None and self.testdata.label is not None:
-            index2textdict = self.testdata.label.index2text
+            if isinstance(self.testdata.label, ZipDataset):
+                for dataset in self.testdata.label.items:
+                    if isinstance(dataset, TextSequenceDataset):
+                        index2textdict = dataset.index2text
+                        break
+            else:
+                index2textdict = self.traindata.label.index2text
         else:
-            index2textdict = self.traindata.label.index2text
-        return index2textdict[idx]
+            if isinstance(self.traindata.label, ZipDataset):
+                for dataset in self.traindata.label.items:
+                    if isinstance(dataset, TextSequenceDataset):
+                        index2textdict = dataset.index2text
+                        break
+            else:
+                index2textdict = self.traindata.label.index2text
+        if idx in index2textdict:
+            return index2textdict[idx]
+        else:
+            return '[UNK]'
 
     def label2index(self, text_data: str):
         text2indexdict = None
         if self.scenario == 'test' and self.testdata is not None and self.testdata.label is not None:
-            text2indexdict = self.testdata.label.text2index
+            if isinstance(self.testdata.label, ZipDataset):
+                for dataset in self.testdata.label.items:
+                    if isinstance(dataset, TextSequenceDataset):
+                        text2indexdict = dataset.text2index
+                        break
+            else:
+                text2indexdict = self.traindata.label.text2index
         else:
-            text2indexdict = self.traindata.label.text2index
+            if isinstance(self.traindata.label, ZipDataset):
+                for dataset in self.traindata.label.items:
+                    if isinstance(dataset, TextSequenceDataset):
+                        text2indexdict = dataset.text2index
+                        break
+            else:
+                text2indexdict = self.traindata.label.text2index
+
         if text_data in text2indexdict:
             return text2indexdict[text_data]
         else:
-            return text2indexdict['<unknown/>']
+            return text2indexdict['[UNK]']
 
     @property
     def vocabs(self):
@@ -672,6 +789,21 @@ class TextSequenceDataProvider(object):
             return self.traindata.label.vocabs
 
 
+    def __setattr__(self, name: str, value) -> None:
+        object.__setattr__(self, name, value)
+        if name in ['traindata','testdata','sequence_length']:
+            if hasattr(self,'sequence_length') and self.sequence_length is not None:
+                if hasattr(self,'traindata') and isinstance(self.traindata, Iterator):
+                    for ds in self.traindata.get_datasets():
+                        if isinstance(ds, TextSequenceDataset):
+                            ds.sequence_length = self.sequence_length
+                if hasattr(self,'testdata') and isinstance(self.testdata, Iterator):
+                    for ds in self.testdata.get_datasets():
+                        if isinstance(ds, TextSequenceDataset):
+                            ds.sequence_length = self.sequence_length
+
+
+
 
 
 
