@@ -1089,7 +1089,7 @@ class AdjustGamma(VisionTransform):
         if self.gamma == 1:
             return image
         image = image.astype(np.float32)
-        image = 255. * self.gain * np.power(image / 255., self.gamma)
+        image = 255. * self.gain * np.power(np.clip(image / 255.,0,1), self.gamma)
         return image.clip(0, 255).astype(np.float32)
 
     def _apply_coords(self, coords,spec:TensorSpec):
@@ -1141,7 +1141,7 @@ class Blur(VisionTransform):
         else:
             self.ksize = int(self.ksize)
         blur = cv2.GaussianBlur(image, (int(self.ksize), int(self.ksize)),cv2.BORDER_DEFAULT)
-        return blur.clip(0, 255).astype(np.float32)
+        return np.clip(blur.clip(0, 255).astype(np.float32),0,255)
 
     def _apply_coords(self, coords,spec:TensorSpec):
         return coords
@@ -1160,7 +1160,7 @@ class InvertColor(VisionTransform):
         return super().apply(input,spec)
 
     def _apply_image(self, image,spec:TensorSpec):
-            return 255 - image
+            return np.clip(255 - image,0,255)
 
     def _apply_coords(self, coords,spec:TensorSpec):
         return coords
@@ -1190,7 +1190,7 @@ class GrayScale(VisionTransform):
         elif image.ndim == 2 and self.keepdims:
             return cv2.cvtColor(image.astype(np.float32), cv2.COLOR_GRAY2RGB)
         else:
-            return image
+            return np.clip(image,0,255)
 
     def _apply_coords(self, coords,spec:TensorSpec):
         return coords
@@ -1212,7 +1212,7 @@ class ToRGB(VisionTransform):
         if image.ndim == 3:
             pass
         elif image.ndim == 2:
-            image=cv2.cvtColor(image.astype(np.float32), cv2.COLOR_GRAY2RGB)
+            image=np.clip(cv2.cvtColor(image.astype(np.float32), cv2.COLOR_GRAY2RGB),0,255)
         return image
 
     def _apply_coords(self, coords,spec:TensorSpec):
@@ -1431,8 +1431,8 @@ class AdaptiveBinarization(VisionTransform):
         ret=None
         th=127.5
         if self.threshold_type=='otsu':
-            th = threshold_otsu(gray)
-            #ret, th = cv2.threshold(gray, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)
+            #th = threshold_otsu(gray)
+            th,ret = cv2.threshold(gray, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)
         elif self.threshold_type == 'minimum':
             th = threshold_minimum(gray)
         elif self.threshold_type == 'yen':
@@ -1450,7 +1450,8 @@ class AdaptiveBinarization(VisionTransform):
             elif abs(gray.mean() - p90) > abs(gray.mean() - p10) and p90 - p10 > 80:  # white background
                 gray[gray > p90] = 255
                 gray[gray < p10] = 0
-        gray = (gray > th).astype(np.float32) * 255.0
+
+        gray =ret if ret is not None else  (gray > th).astype(np.float32) * 255.0
         if gray.max() - gray.min() < 20:
             return clip(image,0,255).astype(np.float32)
         image=cv2.cvtColor(gray, cv2.COLOR_GRAY2RGB)
@@ -1519,7 +1520,7 @@ class CLAHE(VisionTransform):
         clahe = cv2.createCLAHE(clipLimit=self.clipLimit, tileGridSize=(self.gridsize, self.gridsize))
         lab_planes[0] = clahe.apply(lab_planes[0])
         lab = cv2.merge(lab_planes)
-        return cv2.cvtColor(lab, cv2.COLOR_LAB2RGB)
+        return np.clip(cv2.cvtColor(lab, cv2.COLOR_LAB2RGB),0,255)
 
     def _apply_coords(self, coords,spec:TensorSpec):
         return coords
