@@ -2,7 +2,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-
+import time
 import inspect
 import math
 import os
@@ -20,9 +20,16 @@ import torch.nn.functional as F
 from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
+from trident.data.transform import Transform
+
+from trident.backend.tensorspec import ObjectType, TensorSpec
+
+from trident.models.pretrained_utils import _make_recovery_model_include_top
+
+from trident.backend.opencv_backend import image2array, array2image
 
 from trident.backend.common import *
-from trident.backend.pytorch_backend import Layer, Sequential, get_device
+from trident.backend.pytorch_backend import Layer, Sequential, get_device, fix_layer, load
 from trident.backend.pytorch_ops import *
 from trident.data.image_common import *
 from trident.data.bbox_common import *
@@ -34,9 +41,9 @@ from trident.layers.pytorch_normalizations import get_normalization, BatchNorm2d
 from trident.layers.pytorch_pooling import *
 from trident.optims.pytorch_trainer import *
 from trident.misc.visualization_utils import generate_palette,plot_bbox
-
+from trident.data.vision_transforms import Resize,Normalize
 __all__ = [ 'yolo4_body', 'YoloDetectionModel', 'DarknetConv2D', 'DarknetConv2D_BN_Mish',
-           'DarknetConv2D_BN_Leaky', 'YoloLayer']
+           'DarknetConv2D_BN_Leaky', 'YoloLayer','YoLoV4']
 
 _session = get_session()
 _device =get_device()
@@ -44,6 +51,7 @@ _epsilon = _session.epsilon
 _trident_dir = _session.trident_dir
 
 
+dirname = os.path.join(_trident_dir, 'models')
 
 anchors1 = to_tensor(np.array([12, 16, 19, 36, 40, 28]).reshape(-1, 2),requires_grad=False)
 anchors2 = to_tensor(np.array([36, 75, 76, 55, 72, 146]).reshape(-1, 2),requires_grad=False)
@@ -230,7 +238,7 @@ class YoloLayer(Layer):
 
     def compute_grid_offsets(self, grid_size):
 
-        self.register_buffer('grid', meshgrid(grid_size,grid_size,requires_grad=False).view((1, 1, grid_size, grid_size, 2)).float().detach())
+        self.register_buffer('grid', meshgrid(grid_size,grid_size).view((1, 1, grid_size, grid_size, 2)).float().detach())
         #self.grid=meshgrid(grid_size,grid_size,requires_grad=False).view((1, 1, grid_size, grid_size, 2)).float().detach()
 
         # Calculate offsets for each grid
@@ -279,9 +287,9 @@ class YoloLayer(Layer):
 class YoloDetectionModel(ImageDetectionModel):
     def __init__(self, inputs=None, input_shape=None,output=None):
         super(YoloDetectionModel, self).__init__(inputs, input_shape,output)
-        self.preprocess_flow = [resize((input_shape[-2], input_shape[-1]), True), normalize(0, 255)]
-        self.detection_threshold = 0.7
-        self.iou_threshold = 0.3
+        self.preprocess_flow = [Resize((input_shape[-2], input_shape[-1]), True), Normalize(0, 255)]
+        self.detection_threshold = 0.5
+        self.nms_threshold = 0.3
         self.class_names = None
         self.palette = generate_palette(80)
 
@@ -316,12 +324,12 @@ class YoloDetectionModel(ImageDetectionModel):
         area1 = self.area_of(boxes1[..., :2], boxes1[..., 2:])
         return overlap_area / (area0 + area1 - overlap_area + eps)
 
-    def hard_nms(self, box_scores, iou_threshold, top_k=-1, candidate_size=200):
+    def hard_nms(self, box_scores, nms_threshold, top_k=-1, candidate_size=200):
         """
 
         Args:
             box_scores (N, 5): boxes in corner-form and probabilities.
-            iou_threshold: intersection over union threshold.
+            nms_threshold: intersection over union threshold.
             top_k: keep top_k results. If k <= 0, keep all the results.
             candidate_size: only consider the candidates with the highest scores.
         Returns:
@@ -347,7 +355,7 @@ class YoloDetectionModel(ImageDetectionModel):
             indexes = indexes[:-1]
             rest_boxes = boxes[indexes, :]
             iou = self.iou_of(rest_boxes, np.expand_dims(current_box, axis=0), )
-            indexes = indexes[iou <= iou_threshold]
+            indexes = indexes[iou <= nms_threshold]
 
         return box_scores[picked, :], picked
 
@@ -424,19 +432,26 @@ class YoloDetectionModel(ImageDetectionModel):
             try:
                 self._model.to(self.device)
                 self._model.eval()
+                if self._model.input_spec is None:
+                    self._model.input_spec=TensorSpec(shape=TensorShape([None,3,608,608]),object_type=ObjectType.rgb)
+                if self._model.input_spec.object_type is None:
+                    self._model.input_spec.object_type = ObjectType.rgb
                 img = image2array(img)
                 if img.shape[-1] == 4:
                     img = img[:, :, :3]
                 img_orig = img.copy()
 
+                rescale_scale = 1
                 for func in self.preprocess_flow:
-                    if inspect.isfunction(func):
-                        img = func(img)
-                        if func.__qualname__ == 'resize.<locals>.img_op':
-                            scale = func.scale
+                    if (inspect.isfunction(func) or isinstance(func, Transform)) and func is not image_backend_adaption:
+                        img = func(img, spec=self._model.input_spec)
+                        if (inspect.isfunction(func) and func.__qualname__ == 'resize.<locals>.img_op') or (isinstance(func, Transform) and func.name == 'resize'):
+                            rescale_scale = func.scale
 
                 img = image_backend_adaption(img)
-                inp = to_tensor(np.expand_dims(img, 0)).to(self.device).to(self._model.weights[0].data.dtype)
+                inp = to_tensor(np.expand_dims(img, 0)).to(
+                    torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu")).to(
+                    self._model.weights[0].data.dtype)
 
                 if verbose:
                     print("======== data preprocess time:{0:.5f}".format((time.time() - time_time)))
@@ -456,9 +471,9 @@ class YoloDetectionModel(ImageDetectionModel):
                     boxes = concate([xywh2xyxy(boxes[:, :4]), boxes[:, 4:]], axis=-1)
                     boxes = to_numpy(boxes)
                     if len(boxes) > 1:
-                        box_probs, keep = self.hard_nms(boxes[:, :5], iou_threshold=self.iou_threshold, top_k=-1, )
+                        box_probs, keep = self.hard_nms(boxes[:, :5], nms_threshold=self.nms_threshold, top_k=-1, )
                         boxes = boxes[keep]
-                        print('         iou threshold:{0}'.format(self.iou_threshold))
+                        print('         iou threshold:{0}'.format(self.nms_threshold))
                         print('         {0} bboxes keep!'.format(len(boxes)))
                     boxes[:, :4] /=scale
                     boxes[:, :4]=np.round(boxes[:, :4],0)
@@ -515,7 +530,16 @@ def YoLoV4(pretrained=True,
              classes=80,
              **kwargs):
     detector = YoloDetectionModel(input_shape=input_shape, output=yolo4_body(classes, input_shape[-1]))
-    detector.load_model('Models/pretrained_yolov4_mscoco.pth.tar')
+
+    if pretrained:
+        download_model_from_google_drive('1CcbyinE8gQFjMjt05arSg2W0LLUwjsdt', dirname, 'pretrained_yolov4_mscoco.pth')
+        recovery_model = fix_layer(load(os.path.join(dirname, 'pretrained_yolov4_mscoco.pth')))
+        detector.model = recovery_model
+
+    detector.model .input_shape = input_shape
+    detector.model .to(get_device())
+    return detector
+
 
 
 def darknet_body():
@@ -528,101 +552,5 @@ def darknet_body():
         resblock_body(1024, 4)
     )
 
-def yolo4_body(num_classes=80,image_size=608):
-    anchors1 = to_tensor(np.array([12, 16, 19, 36, 40, 28]).reshape(-1, 2),requires_grad=False)
-    anchors2 = to_tensor(np.array([36, 75, 76, 55, 72, 146]).reshape(-1, 2),requires_grad=False)
-    anchors3 = to_tensor(np.array([142, 110, 192, 243, 459, 401]).reshape(-1, 2),requires_grad=False)
-    num_anchors=len(anchors1)
-    """Create YOLO_V4 model CNN body in Keras."""
-    return Sequential(
-            DarknetConv2D_BN_Mish((3, 3), 32,name='first_layer'),
-            resblock_body(64, 1, all_narrow=False,name='block64'),
-            resblock_body(128, 2,name='block128'),
-            resblock_body(256, 8,name='block256'),
-            ShortCut2d(
-                {
-                    1:Sequential(
-                        resblock_body(512, 8,name='block512'),
-                        ShortCut2d(
-                            {
-                                1:Sequential(
-                                    resblock_body(1024, 4, name='block1024'),
-                                    DarknetConv2D_BN_Leaky( (1,1), 512,name='pre_maxpool1'),
-                                    DarknetConv2D_BN_Leaky( (3, 3),1024,name='pre_maxpool2'),
-                                    DarknetConv2D_BN_Leaky((1,1),512,name='pre_maxpool3'),
-                                    ShortCut2d(
-                                        MaxPool2d((13,13),strides=(1,1),auto_pad=True),
-                                        MaxPool2d((9,9), strides=(1, 1), auto_pad=True),
-                                        MaxPool2d((5,5), strides=(1, 1), auto_pad=True),
-                                        Identity(),
-                                        mode='concate'
-                                    ),
-                                    DarknetConv2D_BN_Leaky((1, 1), 512,name='pre_y19_1'),
-                                    DarknetConv2D_BN_Leaky((3, 3), 1024,name='pre_y19_2'),
-                                    DarknetConv2D_BN_Leaky((1, 1), 512,name='y_19',keep_output=True),
-                                    DarknetConv2D_BN_Leaky((1, 1),256,name='pre_y19_upsample'),
-                                    Upsampling2d(scale_factor=2,name='y19_upsample'),
-                                ),
-                                0:DarknetConv2D_BN_Leaky((1, 1), 256)
-                            },mode='concate'),
-                        DarknetConv2D_BN_Leaky((1, 1),256,name='pre_y38_1'),
-                        DarknetConv2D_BN_Leaky((3, 3),512,name='pre_y38_2'),
-                        DarknetConv2D_BN_Leaky((1, 1),256,name='pre_y38_3'),
-                        DarknetConv2D_BN_Leaky((3, 3),512,name='pre_y38_4'),
-                        DarknetConv2D_BN_Leaky((1, 1),256,name='y_38',keep_output=True),
-                        DarknetConv2D_BN_Leaky((1, 1),128,name='pre_y_38_upsample'),
-                        Upsampling2d(scale_factor=2,name='y_38_upsample'),
-                    ),
-                    0:DarknetConv2D_BN_Leaky((1, 1), 128)
-                },
-                mode='concate'),
-            DarknetConv2D_BN_Leaky((1, 1), 128,name='pre_y76_concate1'),
-            DarknetConv2D_BN_Leaky((3, 3), 256,name='pre_y76_concate2'),
-            DarknetConv2D_BN_Leaky((1, 1), 128,name='pre_y76_concate3'),
-            DarknetConv2D_BN_Leaky((3, 3), 256,name='pre_y76_concate4'),
-            DarknetConv2D_BN_Leaky((1, 1), 128,name='pre_y76_concate5'),
-            ShortCut2d(
-                #y76_output
-                Sequential(
-                    DarknetConv2D_BN_Leaky( (3, 3),256,name='pre_y76_output'),
-                    DarknetConv2D( (1, 1),num_anchors * (num_classes + 5),use_bias=True,name='y76_output'),
-                    YoloLayer(anchors=anchors1,num_classes=num_classes,grid_size=76, img_dim=image_size),
-                name='y76_output'),
-                # y38_output
-                Sequential(
-                    ShortCut2d(
-                        DarknetConv2D_BN_Leaky((3, 3), 256, strides=(2, 2), auto_pad=False, padding=((1, 0), (1, 0)),name='y76_downsample'),
-                        branch_from='y_38',mode='concate'),
-                    DarknetConv2D_BN_Leaky((1, 1), 256,name='pre_y38_concate1'),
-                    DarknetConv2D_BN_Leaky((3, 3), 512,name='pre_y38_concate2'),
-                    DarknetConv2D_BN_Leaky((1, 1), 256,name='pre_y38_concate3'),
-                    DarknetConv2D_BN_Leaky((3, 3), 512,name='pre_y38_concate4'),
-                    DarknetConv2D_BN_Leaky((1, 1), 256,name='pre_y38_concate5'),
-                    ShortCut2d(
-                        Sequential(
-                            DarknetConv2D_BN_Leaky((3, 3), 512, name='pre_y38_output'),
-                            DarknetConv2D((1, 1), num_anchors * (num_classes + 5), use_bias=True, name='y38_output'),
-                            YoloLayer(anchors=anchors2, num_classes=num_classes,grid_size=38,  img_dim=image_size),
-                            name='y38_output'),
-
-                        Sequential(
-                            ShortCut2d(
-                                DarknetConv2D_BN_Leaky((3, 3), 512, strides=(2, 2), auto_pad=False, padding=((1, 0), (1, 0)),name='y38_downsample'),
-                                branch_from='y_19', mode='concate'),
-                            DarknetConv2D_BN_Leaky((1, 1), 512,name='pre_y19_concate1'),
-                            DarknetConv2D_BN_Leaky((3, 3), 1024,name='pre_y19_concate2'),
-                            DarknetConv2D_BN_Leaky((1, 1), 512,name='pre_y19_concate3'),
-                            DarknetConv2D_BN_Leaky((3, 3), 1024,name='pre_y19_concate4'),
-                            DarknetConv2D_BN_Leaky((1, 1), 512,name='pre_y19_concate5'),
-                            Sequential(
-                                DarknetConv2D_BN_Leaky((3, 3),1024,name='pre_y19_output'),
-                                DarknetConv2D((1, 1), num_anchors * (num_classes + 5),use_bias=True,name='y19_output'),
-                                YoloLayer(anchors=anchors3,num_classes=num_classes,grid_size=19, img_dim=image_size),
-                            name='y19_output')),
-
-                        mode='concate')
-                )
-                ,mode = 'concate')
-    )
 
 
