@@ -145,6 +145,32 @@ def ResNet(block, layers, input_shape=(224, 224,3), num_classes=1000, use_bias=F
 #         input_shape=(3, 224, 224)
 #     resnet18 = ResNet(basic_block, [2, 2, 2, 2], input_shape, model_name='resnet18')
 
+def ResNet18(include_top=True,
+             pretrained=True,
+             freeze_features=False,
+             input_shape=None,
+             classes=1000,
+             **kwargs):
+    if input_shape is not None and len(input_shape)==3:
+        input_shape=tuple(input_shape)
+    else:
+        input_shape=(3, 224, 224)
+    resnet18 = ResNet(basic_block, [2, 2, 2, 2], input_shape, use_bias=False,include_top=include_top,model_name='resnet18')
+
+    with tf.device(get_device()):
+        if pretrained:
+            download_model_from_google_drive('1vReSW_l8fldyYQ6ay5HCYFGoMaGbdW2T', dirname, 'resnet50_tf.pth')
+            recovery_model = load(os.path.join(dirname, 'resnet50_tf.pth'))
+            recovery_model = fix_layer(recovery_model)
+            recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+            resnet18.model = recovery_model
+        else:
+            resnet18.model = _make_recovery_model_include_top(resnet18.model, include_top=include_top, classes=classes, freeze_features=False)
+
+        resnet18.model.input_shape = input_shape
+    return resnet18
+
+
 def ResNet50(include_top=True,
              pretrained=True,
              freeze_features=False,
@@ -157,7 +183,7 @@ def ResNet50(include_top=True,
         input_shape=(224, 224,3)
     resnet50 =ResNet(bottleneck, [3, 4, 6, 3], input_shape,num_classes=classes,include_top=include_top, model_name='resnet50')
     with tf.device(get_device()):
-        if pretrained==True:
+        if pretrained:
             download_model_from_google_drive('1vReSW_l8fldyYQ6ay5HCYFGoMaGbdW2T',dirname,'resnet50_tf.pth')
             recovery_model=load(os.path.join(dirname,'resnet50_tf.pth'))
             recovery_model = fix_layer(recovery_model)
